@@ -125,21 +125,22 @@ class rcube_vcard
             $this->raw = self::charset_convert($this->raw);
         }
         // vcard has encoded values and charset should be detected
-        else if (
-            $detect && self::$values_decoded
-            && ($detected_charset = self::detect_encoding(self::vcard_encode($this->raw)))
-            && $detected_charset != RCUBE_CHARSET
-        ) {
-            $this->raw = self::charset_convert($this->raw, $detected_charset);
+        else if (self::$values_decoded) {
+            if ($detect) {
+                $charset = self::detect_encoding(self::vcard_encode($this->raw));
+            }
+            if ($charset != RCUBE_CHARSET) {
+                $this->raw = self::charset_convert($this->raw, $charset);
+            }
         }
 
         // find well-known address fields
-        $this->displayname  = isset($this->raw['FN'][0][0]) ? $this->raw['FN'][0][0] : null;
-        $this->surname      = isset($this->raw['N'][0][0]) ? $this->raw['N'][0][0] : null;
-        $this->firstname    = isset($this->raw['N'][0][1]) ? $this->raw['N'][0][1] : null;
-        $this->middlename   = isset($this->raw['N'][0][2]) ? $this->raw['N'][0][2] : null;
-        $this->nickname     = isset($this->raw['NICKNAME'][0][0]) ? $this->raw['NICKNAME'][0][0] : null;
-        $this->organization = isset($this->raw['ORG'][0][0]) ? $this->raw['ORG'][0][0] : null;
+        $this->displayname  = $this->raw['FN'][0][0] ?? '';
+        $this->surname      = $this->raw['N'][0][0] ?? '';
+        $this->firstname    = $this->raw['N'][0][1] ?? '';
+        $this->middlename   = $this->raw['N'][0][2] ?? '';
+        $this->nickname     = $this->raw['NICKNAME'][0][0] ?? '';
+        $this->organization = $this->raw['ORG'][0][0] ?? '';
         $this->business     = (isset($this->raw['X-ABSHOWAS'][0][0]) && $this->raw['X-ABSHOWAS'][0][0] == 'COMPANY')
             || (!empty($this->organization) && isset($this->raw['N'][0]) && @implode('', (array) $this->raw['N'][0]) === '');
 
@@ -175,7 +176,7 @@ class rcube_vcard
 
         // copy name fields to output array
         foreach (['firstname', 'surname', 'middlename', 'nickname', 'organization'] as $col) {
-            if (strlen($this->$col)) {
+            if (is_string($this->$col) && strlen($this->$col)) {
                 $out[$col] = $this->$col;
             }
         }
@@ -200,7 +201,7 @@ class rcube_vcard
                     $key     = $col;
                     $subtype = '';
 
-                    if (!empty($raw['type'])) {
+                    if (!empty($raw['type']) && is_array($raw['type'])) {
                         $raw['type'] = array_map('strtolower', $raw['type']);
 
                         $combined = implode(',', array_diff($raw['type'], ['internet', 'pref']));
@@ -253,7 +254,21 @@ class rcube_vcard
 
                     // split ADR values into assoc array
                     if ($tag == 'ADR') {
-                        list(,, $value['street'], $value['locality'], $value['region'], $value['zipcode'], $value['country']) = $raw;
+                        if (isset($raw[2])) {
+                            $value['street'] = $raw[2];
+                        }
+                        if (isset($raw[3])) {
+                            $value['locality'] = $raw[3];
+                        }
+                        if (isset($raw[4])) {
+                            $value['region'] = $raw[4];
+                        }
+                        if (isset($raw[5])) {
+                            $value['zipcode'] = $raw[5];
+                        }
+                        if (isset($raw[6])) {
+                            $value['country'] = $raw[6];
+                        }
                         $out[$key][] = $value;
                     }
                     // support vCard v4 date format (YYYYMMDD)
@@ -337,7 +352,7 @@ class rcube_vcard
     public function set($field, $value, $type = 'HOME')
     {
         $field   = strtolower($field);
-        $type_uc = strtoupper($type);
+        $type_uc = strtoupper((string) $type);
 
         switch ($field) {
         case 'name':
@@ -414,11 +429,11 @@ class rcube_vcard
                 $value = [
                     '',
                     '',
-                    isset($value['street']) ? $value['street'] : '',
-                    isset($value['locality']) ? $value['locality'] : '',
-                    isset($value['region']) ? $value['region'] : '',
-                    isset($value['zipcode']) ? $value['zipcode'] : '',
-                    isset($value['country']) ? $value['country'] : '',
+                    $value['street'] ?? '',
+                    $value['locality'] ?? '',
+                    $value['region'] ?? '',
+                    $value['zipcode'] ?? '',
+                    $value['country'] ?? '',
                 ];
             }
 
@@ -435,7 +450,7 @@ class rcube_vcard
             if (!empty(self::$fieldmap[$field])) {
                 $tag = self::$fieldmap[$field];
 
-                if (is_array($value) || strlen($value)) {
+                if (is_array($value) || (is_string($value) && strlen($value))) {
                     $this->raw[$tag][] = (array) $value;
                     if ($type) {
                         $index    = count($this->raw[$tag]) - 1;
@@ -542,12 +557,7 @@ class rcube_vcard
     {
         $out = [];
 
-        // check if charsets are specified (usually vcard version < 3.0 but this is not reliable)
-        if (preg_match('/charset=/i', substr($data, 0, 2048))) {
-            $charset = null;
-        }
-        // detect charset and convert to utf-8
-        else if (($charset = self::detect_encoding($data)) && $charset != RCUBE_CHARSET) {
+        if (($charset = self::detect_encoding($data)) && $charset != RCUBE_CHARSET) {
             $data = rcube_charset::convert($data, $charset);
             $data = preg_replace(['/^[\xFE\xFF]{2}/', '/^\xEF\xBB\xBF/', '/^\x00+/'], '', $data); // also remove BOM
             $charset = RCUBE_CHARSET;
@@ -565,7 +575,8 @@ class rcube_vcard
 
             if (preg_match('/^END:VCARD$/i', $line)) {
                 // parse vcard
-                $obj = new rcube_vcard($vcard_block, $charset, true, self::$fieldmap);
+                $obj = new rcube_vcard($vcard_block, $charset, false, self::$fieldmap);
+
                 // FN and N is required by vCard format (RFC 2426)
                 // on import we can be less restrictive, let's addressbook decide
                 if (!empty($obj->displayname) || !empty($obj->surname) || !empty($obj->firstname) || !empty($obj->email)) {
@@ -825,17 +836,22 @@ class rcube_vcard
     {
         $vcard = '';
 
+        // Make sure FN is not empty (required by RFC2426)
+        if (empty($data['FN'])) {
+            $data['FN'] = !empty($data['EMAIL'][0][0]) ? $data['EMAIL'][0][0] : '';
+        }
+
+        // Make sure N is not empty (required by RFC2426)
+        if (empty($data['N'])) {
+            $data['N'] = [['', '', '', '', '']];
+        }
+
+        // Valid N has 5 properties
+        while (isset($data['N'][0]) && is_array($data['N'][0]) && count($data['N'][0]) < 5) {
+            $data['N'][0][] = '';
+        }
+
         foreach ((array)$data as $type => $entries) {
-            // valid N has 5 properties
-            while ($type == "N" && is_array($entries[0]) && count($entries[0]) < 5) {
-                $entries[0][] = "";
-            }
-
-            // make sure FN is not empty (required by RFC2426)
-            if ($type == "FN" && empty($entries) && !empty($data['EMAIL'][0][0])) {
-                $entries[0] = $data['EMAIL'][0][0];
-            }
-
             foreach ((array)$entries as $entry) {
                 $attr = '';
                 if (is_array($entry)) {
@@ -861,7 +877,17 @@ class rcube_vcard
                         }
                         else {
                             foreach ((array)$attrvalues as $attrvalue) {
-                                $attr .= strtoupper(";$attrname=") . self::vcard_quote($attrvalue, ',');
+                                $attrname = strtoupper($attrname);
+                                // TYPE=OTHER is non-standard, TYPE=INTERNET is redundant, remove these
+                                if ($attrname == 'TYPE') {
+                                    $attrvalue = array_map('strtolower', (array) $attrvalue);
+                                    $attrvalue = array_diff($attrvalue, ['other', 'internet']);
+                                    if (empty($attrvalue)) {
+                                        continue;
+                                    }
+                                }
+
+                                $attr .= ';' . $attrname . '=' . self::vcard_quote($attrvalue, ',');
                             }
                         }
                     }
@@ -870,8 +896,8 @@ class rcube_vcard
                     $value = $entry;
                 }
 
-                // skip empty entries
-                if (self::is_empty($value)) {
+                // Skip empty entries that aren't required by vCard v3 format
+                if (!in_array($type, ['N', 'FN']) && self::is_empty($value)) {
                     continue;
                 }
 
@@ -975,7 +1001,7 @@ class rcube_vcard
     }
 
     /**
-     * Returns UNICODE type based on BOM (Byte Order Mark)
+     * Returns character set of a vCard input
      *
      * @param string $string Input string to test
      *
@@ -983,8 +1009,67 @@ class rcube_vcard
      */
     private static function detect_encoding($string)
     {
-        $fallback = rcube::get_instance()->config->get('default_charset', 'ISO-8859-1'); // fallback to Latin-1
+        // Extract the plain text from the vCard, so the detection is more accurate
+        // This will for example exclude photos
 
-        return rcube_charset::detect($string, $fallback);
+        // Perform RFC2425 line unfolding and split lines
+        $string = preg_replace(["/\r/", "/\n\s+/"], '', $string);
+        $lines  = explode("\n", $string);
+        $string = '';
+
+        for ($i = 0, $len = count($lines); $i < $len; $i++) {
+            if (!($pos = strpos($lines[$i], ':'))) {
+                continue;
+            }
+
+            $prefix = substr($lines[$i], 0, $pos);
+
+            // We remove \0 as so it works with UTF-16/UTF-32 encodings
+            $prefix = str_replace("\0", '', $prefix);
+
+            // Take only properties that are known to contain human-readable text
+            if (!preg_match('/^(item\d+\.)?(N|FN|ORG|ADR|NOTE|TITLE|CATEGORIES)(;|$)/', $prefix)) {
+                continue;
+            }
+
+            $data = substr($lines[$i], $pos + 1);
+
+            if (preg_match('/;CHARSET=([a-z0-9-]+)/i', $prefix, $matches)) {
+                // We assume there's only one charset in the input
+                return $matches[1];
+            }
+
+            $matches = null;
+            $enc = null;
+
+            if (stripos($prefix, 'base64') || preg_match('/ENCODING=(QUOTED-PRINTABLE|B|BASE64)/i', $prefix, $matches)) {
+                $enc = $matches ? strtoupper($matches[1]) : 'BASE64';
+                // add next line(s) to value string if QP line end detected
+                if ($enc == 'QUOTED-PRINTABLE') {
+                    while (preg_match('/=$/', $lines[$i])) {
+                        $data .= "\n" . $lines[++$i];
+                    }
+                }
+
+                $data = self::decode_value($data, $enc);
+            }
+
+            if (!$enc || $enc == 'QUOTED-PRINTABLE') {
+                $data = self::vcard_unquote($data);
+            }
+
+            if (is_array($data)) {
+                $data = implode(' ', array_filter($data));
+            }
+
+            $string .= $data . ' ';
+
+            // 100 KB should be enough for charset check
+            if (strlen($string) > 100 * 1024) {
+                break;
+            }
+        }
+
+        return rcube_charset::check($string) ?: RCUBE_CHARSET;
     }
 }
